#include "minibeamLib.h"

int findMinFloat(const float *values, int startIdx, size_t valuesSize);
int findMaxFloat(const float *values, int startIdx, size_t valuesSize);
int computeCenterOfMassIgnoringBackground(const double *positions,
  const float *values, int startIdx, size_t positionsAndValuesSize,
  float background, double *centerOfMass);

/*
 * Sets optimalPosition to computed optimal position using method specified by
 * opdm.
 *
 * Returns 0 if OK, less than 0 if error.
 */
int minibeamComputeOptimalPosition(int opdm, const double *positions,
    const float *values, int startIdx, size_t positionsAndValuesSize,
    double *optimalPosition) {
  int indexOfMax;
  int indexOfMin;
  int status;

  if (startIdx < 0) return -1;
  if (positionsAndValuesSize - startIdx <= 1) return -2;

  if (opdm == MINIBEAM_OPDM_MAX) {
    indexOfMax = findMaxFloat(values, startIdx,
      positionsAndValuesSize);
    if (indexOfMax < 0) return -3;
    *optimalPosition = positions[indexOfMax];
  } else if (opdm == MINIBEAM_OPDM_MINUS_MIN__CNTR_MASS) {
    indexOfMin = findMinFloat(values, startIdx,
      positionsAndValuesSize);
    if (indexOfMin < 0) indexOfMin = startIdx;
    status = computeCenterOfMassIgnoringBackground(positions, values,
      startIdx, positionsAndValuesSize, values[indexOfMin], optimalPosition);
    if (status < 0) return -4;
  } else {
    return -5;
  }
  return 0;
}

/*
 * Finds index of minimum value in values.
 *
 * Returns index of minimum value in values starting at startIdx, or -1 if
 * startIdx < 0, valuesSize - startIdx <= 1, or all values are equal.
 */
int findMinFloat(const float *values, int startIdx, size_t valuesSize) {
  int minIdx;
  int i;

  if (startIdx < 0) return -1;
  if (valuesSize - startIdx <= 1) return -1;
  minIdx = startIdx;
  for (i = startIdx + 1; i < valuesSize; i++)
    if (values[i] < values[minIdx]) minIdx = i;
  for (i = startIdx + 1; i < valuesSize; i++)
    if (values[i] != values[startIdx]) return minIdx;
  return -1;
}

/*
 * Finds index of maximum value in values.
 *
 * Returns index of maximum value in values starting at startIdx, or -1 if
 * startIdx < 0, valuesSize - startIdx <= 1, or all values are equal.
 */
int findMaxFloat(const float *values, int startIdx, size_t valuesSize) {
  int maxIdx;
  int i;

  if (startIdx < 0) return -1;
  if (valuesSize - startIdx <= 1) return -1;
  maxIdx = startIdx;
  for (i = startIdx + 1; i < valuesSize; i++)
    if (values[i] > values[maxIdx]) maxIdx = i;
  for (i = startIdx + 1; i < valuesSize; i++)
    if (values[i] != values[startIdx]) return maxIdx;
  return -1;
}

/*
 * Sets centerOfMass to center of mass after subtracting out background.
 *
 * Returns 0 if OK, -1 if error.
 *
 * Based on discussion with, and equation from, Rong Huang (IMCA-CAT).
 *
 * Ref: http://mathworld.wolfram.com/GeometricCentroid.html
 * Ref: http://en.wikipedia.org/wiki/Center_of_mass
 */
int computeCenterOfMassIgnoringBackground(const double *positions,
    const float *values, int startIdx, size_t positionsAndValuesSize,
    float background, double *centerOfMass) {
  double weightedSum;
  double sum;
  int i;

  if (startIdx < 0) return -1;
  if (positionsAndValuesSize - startIdx < 1) return -1;
  weightedSum = 0.0;
  sum = 0.0;
  for (i = startIdx + 1; i < positionsAndValuesSize; i++) {
    weightedSum += positions[i] * (values[i] - background);
    sum += values[i] - background;
  }
  if (sum == 0.0) return -1;
  *centerOfMass = weightedSum / sum;
  return 0;
}
